@@ -16,10 +16,16 @@ spec_result <- c(
 # Helpers -----------------------------------------------------------------
 
 sql_union <- function(..., .order_by = NULL, .ctx) {
-  query <- .ctx$tweaks$union(c(...))
+  queries <- c(...)
+  if (length(queries) == 1) {
+    query <- queries
+  } else {
+    stopifnot(!is.null(.ctx))
+    query <- .ctx$tweaks$union(queries)
+  }
 
-  if (!missing(.order_by)) {
-    query <- paste(query, "ORDER BY", .order_by)
+  if (!is.null(.order_by)) {
+    query <- paste0(query, " ORDER BY ", .order_by)
   }
   query
 }
@@ -29,6 +35,8 @@ trivial_statement <- function(ctx, table_name) {
 }
 
 trivial_query <- function(n = 1L, column = "a", .order_by = NULL, .ctx = NULL) {
+  # Zero-row queries are hard-coded, search for 1 = 0
+  stopifnot(n > 0)
   value <- trivial_values(n)
   if (length(column) == n) {
     query <- paste0("SELECT ", paste0(value, " AS ", column, collapse = ", "))
