@@ -1,7 +1,8 @@
 #' spec_sql_write_table
+#' @family sql specifications
 #' @usage NULL
 #' @format NULL
-#' @keywords internal
+#' @keywords NULL
 #' @importFrom lubridate with_tz
 spec_sql_write_table <- list(
   write_table_formals = function() {
@@ -9,14 +10,16 @@ spec_sql_write_table <- list(
     expect_equal(names(formals(dbWriteTable)), c("conn", "name", "value", "..."))
   },
 
-  #' @return
-  #' `dbWriteTable()` returns `TRUE`, invisibly.
   write_table_return = function(con, table_name) {
+    #' @return
+    #' `dbWriteTable()` returns `TRUE`, invisibly.
     expect_invisible_true(dbWriteTable(con, table_name, data.frame(a = 1L)))
   },
 
-  #' If the table exists, and both `append` and `overwrite` arguments are unset,
+  #'
   write_table_overwrite = function(con, table_name) {
+    #' @section Failure modes:
+    #' If the table exists, and both `append` and `overwrite` arguments are unset,
     test_in <- data.frame(a = 1L)
     dbWriteTable(con, table_name, test_in)
     expect_error(dbWriteTable(con, table_name, data.frame(a = 2L)))
@@ -25,10 +28,10 @@ spec_sql_write_table <- list(
     expect_equal_df(test_out, test_in)
   },
 
-  #' or `append = TRUE` and the data frame with the new data has different
-  #' column names,
-  #' an error is raised; the remote table remains unchanged.
   write_table_append_incompatible = function(con, table_name) {
+    #' or `append = TRUE` and the data frame with the new data has different
+    #' column names,
+    #' an error is raised; the remote table remains unchanged.
     test_in <- data.frame(a = 1L)
     dbWriteTable(con, table_name, test_in)
     expect_error(dbWriteTable(con, table_name, data.frame(b = 2L), append = TRUE))
@@ -38,22 +41,22 @@ spec_sql_write_table <- list(
   },
 
   #'
-  #' An error is raised when calling this method for a closed
   write_table_closed_connection = function(ctx, closed_con) {
+    #' An error is raised when calling this method for a closed
     expect_error(dbWriteTable(closed_con, "test", data.frame(a = 1)))
   },
 
-  #' or invalid connection.
   write_table_invalid_connection = function(ctx, invalid_con) {
+    #' or invalid connection.
     expect_error(dbWriteTable(invalid_con, "test", data.frame(a = 1)))
   },
 
-  #' An error is also raised
   write_table_error = function(ctx, con, table_name) {
+    #' An error is also raised
     test_in <- data.frame(a = 1L)
-    #' if `name` cannot be processed with [dbQuoteIdentifier()]
+    #' if `name` cannot be processed with [dbQuoteIdentifier()] or
     expect_error(dbWriteTable(con, NA, test_in))
-    #' or if this results in a non-scalar.
+    #' if this results in a non-scalar.
     expect_error(dbWriteTable(con, c(table_name, table_name), test_in))
 
     #' Invalid values for the additional arguments `row.names`,
@@ -104,9 +107,9 @@ spec_sql_write_table <- list(
   #' They must be provided as named arguments.
   #' See the "Specification" and "Value" sections for details on their usage.
 
-  #' @section Specification:
-  #' The `name` argument is processed as follows,
   write_table_name = function(ctx, con) {
+    #' @section Specification:
+    #' The `name` argument is processed as follows,
     #' to support databases that allow non-syntactic names for their objects:
     if (isTRUE(ctx$tweaks$strict_identifier)) {
       table_names <- "a"
@@ -116,188 +119,281 @@ spec_sql_write_table <- list(
 
     for (table_name in table_names) {
       test_in <- data.frame(a = 1)
-      with_remove_test_table(name = dbQuoteIdentifier(con, table_name), {
-        #' - If an unquoted table name as string: `dbWriteTable()` will do the quoting,
-        dbWriteTable(con, table_name, test_in)
-        test_out <- check_df(dbReadTable(con, dbQuoteIdentifier(con, table_name)))
-        expect_equal_df(test_out, test_in)
-        #'   perhaps by calling `dbQuoteIdentifier(conn, x = name)`
-      })
+      local_remove_test_table(con, table_name)
+      #' - If an unquoted table name as string: `dbWriteTable()` will do the quoting,
+      dbWriteTable(con, table_name, test_in)
+      test_out <- check_df(dbReadTable(con, dbQuoteIdentifier(con, table_name)))
+      expect_equal_df(test_out, test_in)
+      #'   perhaps by calling `dbQuoteIdentifier(conn, x = name)`
+    }
+  },
 
-      with_remove_test_table(name = dbQuoteIdentifier(con, table_name), {
-        #' - If the result of a call to [dbQuoteIdentifier()]: no more quoting is done
-        dbWriteTable(con, dbQuoteIdentifier(con, table_name), test_in)
-        test_out <- check_df(dbReadTable(con, table_name))
-        expect_equal_df(test_out, test_in)
-      })
+  write_table_name_quoted = function(ctx, con) {
+    #' - If the result of a call to [dbQuoteIdentifier()]: no more quoting is done
+    skip_if_not_dbitest(ctx, "1.7.2")
+
+    if (isTRUE(ctx$tweaks$strict_identifier)) {
+      table_names <- "a"
+    } else {
+      table_names <- c("a", "with spaces", "with,comma")
+    }
+
+    for (table_name in table_names) {
+      test_in <- data.frame(a = 1)
+
+      local_remove_test_table(con, table_name)
+      dbWriteTable(con, dbQuoteIdentifier(con, table_name), test_in)
+      test_out <- check_df(dbReadTable(con, table_name))
+      expect_equal_df(test_out, test_in)
     }
   },
 
   #'
-  #' If the `overwrite` argument is `TRUE`, an existing table of the same name
-  #' will be overwritten.
+  write_table_value_df = function(con, table_name) {
+    #' The `value` argument must be a data frame
+    test_in <- trivial_df()
+    dbWriteTable(con, table_name, test_in)
+
+    test_out <- check_df(dbReadTable(con, table_name))
+    expect_equal_df(test_out, test_in)
+  },
+
+  write_table_value_subset = function(ctx, con, table_name) {
+    #' with a subset of the columns of the existing table if `append = TRUE`.
+    test_in <- trivial_df(3, letters[1:3])
+    dbCreateTable(con, table_name, test_in)
+    dbWriteTable(con, table_name, test_in[2], append = TRUE)
+
+    test_out <- check_df(dbReadTable(con, table_name))
+
+    test_in[c(1, 3)] <- NA_real_
+    expect_equal_df(test_out, test_in)
+  },
+
+  write_table_value_shuffle = function(ctx, con, table_name) {
+    #' The order of the columns does not matter with `append = TRUE`.
+    test_in <- trivial_df(3, letters[1:3])
+    dbCreateTable(con, table_name, test_in)
+    dbWriteTable(con, table_name, test_in[c(2, 3, 1)], append = TRUE)
+
+    test_out <- check_df(dbReadTable(con, table_name))
+    expect_equal_df(test_out, test_in)
+  },
+
+  #
+  write_table_value_shuffle_subset = function(ctx, con, table_name) {
+    test_in <- trivial_df(4, letters[1:4])
+    dbCreateTable(con, table_name, test_in)
+    dbWriteTable(con, table_name, test_in[c(4, 1, 3)], append = TRUE)
+
+    test_out <- check_df(dbReadTable(con, table_name))
+
+    test_in[2] <- NA_real_
+    expect_equal_df(test_out, test_in)
+  },
+
+  #'
   overwrite_table = function(ctx, con, table_name) {
-    iris <- get_iris(ctx)
-    dbWriteTable(con, table_name, iris)
+    #' If the `overwrite` argument is `TRUE`, an existing table of the same name
+    #' will be overwritten.
+    penguins <- get_penguins(ctx)
+    dbWriteTable(con, table_name, penguins)
     expect_error(
-      dbWriteTable(con, table_name, iris[1:10, ], overwrite = TRUE),
+      dbWriteTable(con, table_name, penguins[1, ], overwrite = TRUE),
       NA
     )
-    iris_out <- check_df(dbReadTable(con, table_name))
-    expect_equal_df(iris_out, iris[1:10, ])
+    penguins_out <- check_df(dbReadTable(con, table_name))
+    expect_equal_df(penguins_out, penguins[1, ])
   },
 
-  #' This argument doesn't change behavior if the table does not exist yet.
   overwrite_table_missing = function(ctx, con, table_name) {
-    iris_in <- get_iris(ctx)
+    #' This argument doesn't change behavior if the table does not exist yet.
+    penguins_in <- get_penguins(ctx)
     expect_error(
-      dbWriteTable(con, table_name, iris_in[1:10, ], overwrite = TRUE),
+      dbWriteTable(con, table_name, penguins_in[1, ], overwrite = TRUE),
       NA
     )
-    iris_out <- check_df(dbReadTable(con, table_name))
-    expect_equal_df(iris_out, iris_in[1:10, ])
+    penguins_out <- check_df(dbReadTable(con, table_name))
+    expect_equal_df(penguins_out, penguins_in[1, ])
   },
 
   #'
-  #' If the `append` argument is `TRUE`, the rows in an existing table are
-  #' preserved, and the new data are appended.
   append_table = function(ctx, con, table_name) {
-    iris <- get_iris(ctx)
-    dbWriteTable(con, table_name, iris)
-    expect_error(dbWriteTable(con, table_name, iris[1:10, ], append = TRUE), NA)
-    iris_out <- check_df(dbReadTable(con, table_name))
-    expect_equal_df(iris_out, rbind(iris, iris[1:10, ]))
+    #' If the `append` argument is `TRUE`, the rows in an existing table are
+    #' preserved, and the new data are appended.
+    penguins <- get_penguins(ctx)
+    dbWriteTable(con, table_name, penguins)
+    expect_error(dbWriteTable(con, table_name, penguins[1, ], append = TRUE), NA)
+    penguins_out <- check_df(dbReadTable(con, table_name))
+    expect_equal_df(penguins_out, rbind(penguins, penguins[1, ]))
   },
 
-  #' If the table doesn't exist yet, it is created.
   append_table_new = function(ctx, con, table_name) {
-    iris <- get_iris(ctx)
-    expect_error(dbWriteTable(con, table_name, iris[1:10, ], append = TRUE), NA)
-    iris_out <- check_df(dbReadTable(con, table_name))
-    expect_equal_df(iris_out, iris[1:10, ])
+    #' If the table doesn't exist yet, it is created.
+    penguins <- get_penguins(ctx)
+    expect_error(dbWriteTable(con, table_name, penguins[1, ], append = TRUE), NA)
+    penguins_out <- check_df(dbReadTable(con, table_name))
+    expect_equal_df(penguins_out, penguins[1, ])
   },
 
   #'
-  #' If the `temporary` argument is `TRUE`, the table is not available in a
-  #' second connection and is gone after reconnecting.
-  temporary_table = function(ctx, con, table_name = "dbit08") {
+  temporary_table_1 = function(ctx, con, table_name = "dbit08") {
+    #' If the `temporary` argument is `TRUE`, the table is not available in a
+    #' second connection and is gone after reconnecting.
     #' Not all backends support this argument.
     if (!isTRUE(ctx$tweaks$temporary_tables)) {
       skip("tweak: temporary_tables")
     }
 
-    iris <- get_iris(ctx)[1:30, ]
-    dbWriteTable(con, table_name, iris, temporary = TRUE)
-    iris_out <- check_df(dbReadTable(con, table_name))
-    expect_equal_df(iris_out, iris)
+    penguins <- get_penguins(ctx)
+    dbWriteTable(con, table_name, penguins, temporary = TRUE)
+    penguins_out <- check_df(dbReadTable(con, table_name))
+    expect_equal_df(penguins_out, penguins)
 
     con2 <- local_connection(ctx)
     expect_error(dbReadTable(con2, table_name))
   },
   # second stage
-  temporary_table = function(ctx, con) {
+  temporary_table_2 = function(ctx, con) {
     if (!isTRUE(ctx$tweaks$temporary_tables)) {
       skip("tweak: temporary_tables")
     }
 
+    # table_name not in formals on purpose: this means that this table won't be
+    # removed at the end of the test
     table_name <- "dbit08"
     expect_error(dbReadTable(con, table_name))
   },
 
-  #' A regular, non-temporary table is visible in a second connection
-  table_visible_in_other_connection = function(ctx, con) {
-    iris30 <- get_iris(ctx)[1:30, ]
+  table_visible_in_other_connection_1 = function(ctx, local_con) {
+    #' A regular, non-temporary table is visible in a second connection,
+    penguins30 <- get_penguins(ctx)
 
+    # table_name not in formals on purpose: this means that this table won't be
+    # removed at the end of the test
     table_name <- "dbit09"
 
-    dbWriteTable(con, table_name, iris30)
-    iris_out <- check_df(dbReadTable(con, table_name))
-    expect_equal_df(iris_out, iris30)
+    dbWriteTable(local_con, table_name, penguins30)
+    penguins_out <- check_df(dbReadTable(local_con, table_name))
+    expect_equal_df(penguins_out, penguins30)
 
     con2 <- local_connection(ctx)
-    expect_equal_df(dbReadTable(con2, table_name), iris30)
+    expect_equal_df(dbReadTable(con2, table_name), penguins30)
   },
   # second stage
-  table_visible_in_other_connection = function(ctx, con, table_name = "dbit09") {
-    #' and after reconnecting to the database.
-    iris30 <- get_iris(ctx)[1:30, ]
+  table_visible_in_other_connection_2 = function(ctx, con) {
+    #' in a pre-existing connection,
+    penguins30 <- get_penguins(ctx)
 
-    expect_equal_df(check_df(dbReadTable(con, table_name)), iris30)
+    # table_name not in formals on purpose: this means that this table won't be
+    # removed at the end of the test
+    table_name <- "dbit09"
+
+    expect_equal_df(check_df(dbReadTable(con, table_name)), penguins30)
+  },
+  # third stage
+  table_visible_in_other_connection_3 = function(ctx, local_con, table_name = "dbit09") {
+    #' and after reconnecting to the database.
+    penguins30 <- get_penguins(ctx)
+
+    expect_equal_df(check_df(dbReadTable(local_con, table_name)), penguins30)
   },
 
   #'
-  #' SQL keywords can be used freely in table names, column names, and data.
   roundtrip_keywords = function(ctx, con) {
+    #' SQL keywords can be used freely in table names, column names, and data.
     tbl_in <- data.frame(
-      SELECT = "UNIQUE", FROM = "JOIN", WHERE = "ORDER",
+      select = "unique", from = "join", where = "order",
       stringsAsFactors = FALSE
     )
-    test_table_roundtrip(con, tbl_in, name = "EXISTS")
+    test_table_roundtrip(con, tbl_in, name = "exists")
   },
 
-  #' Quotes, commas, and spaces can also be used in the data,
-  #' and, if the database supports non-syntactic identifiers,
-  #' also for table names and column names.
-  roundtrip_quotes = function(ctx, con) {
-    if (!isTRUE(ctx$tweaks$strict_identifier)) {
-      table_names <- c(
-        as.character(dbQuoteIdentifier(con, "")),
-        as.character(dbQuoteString(con, "")),
-        "with space",
-        ","
-      )
-    } else {
-      table_names <- "a"
+  roundtrip_quotes = function(ctx, con, table_name) {
+    #' Quotes, commas, spaces, and other special characters such as newlines and tabs,
+    #' can also be used in the data,
+    tbl_in <- data.frame(
+      as.character(dbQuoteString(con, "")),
+      as.character(dbQuoteIdentifier(con, "")),
+      "with space",
+      "a,b", "a\nb", "a\tb", "a\rb", "a\bb",
+      "a\\Nb", "a\\tb", "a\\rb", "a\\bb", "a\\Zb",
+      stringsAsFactors = FALSE
+    )
+
+    names(tbl_in) <- letters[seq_along(tbl_in)]
+    test_table_roundtrip(con, tbl_in)
+  },
+
+  roundtrip_quotes_table_names = function(ctx, con) {
+    #' and, if the database supports non-syntactic identifiers,
+    #' also for table names
+    if (isTRUE(ctx$tweaks$strict_identifier)) {
+      skip("tweak: strict_identifier")
     }
 
+    table_names <- c(
+      as.character(dbQuoteIdentifier(con, "")),
+      as.character(dbQuoteString(con, "")),
+      "with space",
+      "a,b", "a\nb", "a\tb", "a\rb", "a\bb",
+      "a\\Nb", "a\\tb", "a\\rb", "a\\bb", "a\\Zb"
+    )
+
+    tbl_in <- trivial_df()
+
     for (table_name in table_names) {
-      tbl_in <- data.frame(
-        a = as.character(dbQuoteString(con, "")),
-        b = as.character(dbQuoteIdentifier(con, "")),
-        c = "with space",
-        d = ",",
-        stringsAsFactors = FALSE
-      )
-
-      if (!isTRUE(ctx$tweaks$strict_identifier)) {
-        names(tbl_in) <- c(
-          as.character(dbQuoteIdentifier(con, "")),
-          as.character(dbQuoteString(con, "")),
-          "with space",
-          ","
-        )
-      }
-
-      test_table_roundtrip(con, tbl_in)
+      test_table_roundtrip_one(con, tbl_in, .add_na = "none")
     }
   },
 
+  roundtrip_quotes_column_names = function(ctx, con) {
+    #' and column names.
+    skip_if_not_dbitest(ctx, "1.7.2")
+
+    if (isTRUE(ctx$tweaks$strict_identifier)) {
+      skip("tweak: strict_identifier")
+    }
+
+    column_names <- c(
+      as.character(dbQuoteIdentifier(con, "")),
+      as.character(dbQuoteString(con, "")),
+      "with space",
+      "a,b", "a\nb", "a\tb", "a\rb", "a\bb",
+      "a\\nb", "a\\tb", "a\\rb", "a\\bb", "a\\zb"
+    )
+
+    tbl_in <- trivial_df(length(column_names), column_names)
+
+    test_table_roundtrip_one(con, tbl_in, .add_na = "none")
+  },
+
   #'
-  #' The following data types must be supported at least,
-  #' and be read identically with [dbReadTable()]:
-  #' - integer
   roundtrip_integer = function(ctx, con) {
+    #' The following data types must be supported at least,
+    #' and be read identically with [dbReadTable()]:
+    #' - integer
     tbl_in <- data.frame(a = c(1:5))
     test_table_roundtrip(con, tbl_in)
   },
 
-  #' - numeric
   roundtrip_numeric = function(ctx, con) {
+    #' - numeric
     tbl_in <- data.frame(a = c(seq(1, 3, by = 0.5)))
     test_table_roundtrip(con, tbl_in)
     #'   (the behavior for `Inf` and `NaN` is not specified)
   },
 
-  #' - logical
   roundtrip_logical = function(ctx, con) {
+    #' - logical
     tbl_in <- data.frame(a = c(TRUE, FALSE, NA))
     tbl_exp <- tbl_in
     tbl_exp$a <- ctx$tweaks$logical_return(tbl_exp$a)
     test_table_roundtrip(con, tbl_in, tbl_exp)
   },
 
-  #' - `NA` as NULL
   roundtrip_null = function(ctx, con) {
+    #' - `NA` as NULL
     tbl_in <- data.frame(a = NA)
     test_table_roundtrip(
       con, tbl_in,
@@ -346,27 +442,27 @@ spec_sql_write_table <- list(
     test_table_roundtrip(con, tbl_out, tbl_expected = tbl_out)
   },
 
-  #' - character (in both UTF-8
   roundtrip_character = function(ctx, con) {
+    #' - character (in both UTF-8
     tbl_in <- data.frame(
-      id = seq_along(texts),
-      a = c(texts),
+      id = seq_along(get_texts()),
+      a = get_texts(),
       stringsAsFactors = FALSE
     )
     test_table_roundtrip(con, tbl_in)
   },
 
-  #'   and native encodings),
   roundtrip_character_native = function(ctx, con) {
+    #'   and native encodings),
     tbl_in <- data.frame(
-      a = c(enc2native(texts)),
+      a = c(enc2native(get_texts())),
       stringsAsFactors = FALSE
     )
     test_table_roundtrip(con, tbl_in)
   },
 
-  #'   supporting empty strings
   roundtrip_character_empty = function(ctx, con) {
+    #'   supporting empty strings
     tbl_in <- data.frame(
       a = c("", "a"),
       stringsAsFactors = FALSE
@@ -374,8 +470,8 @@ spec_sql_write_table <- list(
     test_table_roundtrip(con, tbl_in)
   },
 
-  #'   before and after a non-empty string
   roundtrip_character_empty_after = function(ctx, con) {
+    #'   before and after a non-empty string
     tbl_in <- data.frame(
       a = c("a", ""),
       stringsAsFactors = FALSE
@@ -383,18 +479,18 @@ spec_sql_write_table <- list(
     test_table_roundtrip(con, tbl_in)
   },
 
-  #' - factor (returned as character)
   roundtrip_factor = function(ctx, con) {
+    #' - factor (returned as character)
     tbl_in <- data.frame(
-      a = factor(c(texts))
+      a = factor(get_texts())
     )
     tbl_exp <- tbl_in
     tbl_exp$a <- as.character(tbl_exp$a)
     test_table_roundtrip(con, tbl_in, tbl_exp)
   },
 
-  #' - list of raw
   roundtrip_raw = function(ctx, con) {
+    #' - list of raw
     #'   (if supported by the database)
     if (isTRUE(ctx$tweaks$omit_blob_tests)) {
       skip("tweak: omit_blob_tests")
@@ -412,8 +508,8 @@ spec_sql_write_table <- list(
     )
   },
 
-  #' - objects of type [blob::blob]
   roundtrip_blob = function(ctx, con) {
+    #' - objects of type [blob::blob]
     #'   (if supported by the database)
     if (isTRUE(ctx$tweaks$omit_blob_tests)) {
       skip("tweak: omit_blob_tests")
@@ -429,26 +525,53 @@ spec_sql_write_table <- list(
     )
   },
 
-  #' - date
   roundtrip_date = function(ctx, con) {
+    #' - date
     #'   (if supported by the database;
     if (!isTRUE(ctx$tweaks$date_typed)) {
       skip("tweak: !date_typed")
     }
 
-    #'   returned as `Date`)
+    #'   returned as `Date`),
     tbl_in <- data.frame(a = as_numeric_date(c(Sys.Date() + 1:5)))
     test_table_roundtrip(
       con, tbl_in,
       transform = function(tbl_out) {
-        expect_is(unclass(tbl_out$a), "numeric")
+        expect_type(unclass(tbl_out$a), "double")
+        tbl_out
+      }
+    )
+  },
+
+  roundtrip_date_extended = function(ctx, con) {
+    #'   also for dates prior to 1970 or 1900 or after 2038
+    if (!isTRUE(ctx$tweaks$date_typed)) {
+      skip("tweak: !date_typed")
+    }
+
+    tbl_in <- data.frame(a = as_numeric_date(c(
+      "1811-11-11",
+      "1899-12-31",
+      "1900-01-01",
+      "1950-05-05",
+      "1969-12-31",
+      "1970-01-01",
+      "2037-01-01",
+      "2038-01-01",
+      "2040-01-01",
+      "2999-09-09"
+    )))
+    test_table_roundtrip(
+      con, tbl_in,
+      transform = function(tbl_out) {
+        expect_type(unclass(tbl_out$a), "double")
         tbl_out
       }
     )
   },
 
-  #' - time
   roundtrip_time = function(ctx, con) {
+    #' - time
     #'   (if supported by the database;
     if (!isTRUE(ctx$tweaks$time_typed)) {
       skip("tweak: !time_typed")
@@ -474,8 +597,8 @@ spec_sql_write_table <- list(
     )
   },
 
-  #' - timestamp
   roundtrip_timestamp = function(ctx, con) {
+    #' - timestamp
     #'   (if supported by the database;
     if (!isTRUE(ctx$tweaks$timestamp_typed)) {
       skip("tweak: !timestamp_typed")
@@ -491,18 +614,58 @@ spec_sql_write_table <- list(
     attr(local, "tzone") <- ""
     tbl_in <- data.frame(id = seq_along(local))
     tbl_in$local <- local
-    tbl_in$GMT <- lubridate::with_tz(local, tzone = "GMT")
-    tbl_in$PST8PDT <- lubridate::with_tz(local, tzone = "PST8PDT")
-    tbl_in$UTC <- lubridate::with_tz(local, tzone = "UTC")
+    tbl_in$gmt <- lubridate::with_tz(local, tzone = "GMT")
+    tbl_in$pst8pdt <- lubridate::with_tz(local, tzone = "PST8PDT")
+    tbl_in$utc <- lubridate::with_tz(local, tzone = "UTC")
+
+    #'   respecting the time zone but not necessarily preserving the
+    #'   input time zone),
+    test_table_roundtrip(
+      con, tbl_in,
+      transform = function(out) {
+        dates <- map_lgl(out, inherits, "POSIXt")
+        tz <- toupper(names(out))
+        tz[tz == "LOCAL"] <- ""
+        out[dates] <- Map(lubridate::with_tz, out[dates], tz[dates])
+        out
+      }
+    )
+  },
+
+  roundtrip_timestamp_extended = function(ctx, con) {
+    #'   also for timestamps prior to 1970 or 1900 or after 2038
+    if (!isTRUE(ctx$tweaks$timestamp_typed)) {
+      skip("tweak: !timestamp_typed")
+    }
+
+    local <- as.POSIXct(c(
+      "1811-11-11",
+      "1899-12-31",
+      "1900-01-01",
+      "1950-05-05",
+      "1969-12-31",
+      "1970-01-01",
+      "2037-01-01",
+      "2038-01-01",
+      "2040-01-01",
+      "2999-09-09"
+    ))
+
+    attr(local, "tzone") <- ""
+    tbl_in <- data.frame(id = seq_along(local))
+    tbl_in$local <- local
+    tbl_in$gmt <- lubridate::with_tz(local, tzone = "GMT")
+    tbl_in$pst8pdt <- lubridate::with_tz(local, tzone = "PST8PDT")
+    tbl_in$utc <- lubridate::with_tz(local, tzone = "UTC")
 
     #'   respecting the time zone but not necessarily preserving the
     #'   input time zone)
     test_table_roundtrip(
       con, tbl_in,
       transform = function(out) {
-        dates <- vapply(out, inherits, "POSIXt", FUN.VALUE = logical(1L))
-        tz <- names(out)
-        tz[tz == "local"] <- ""
+        dates <- map_lgl(out, inherits, "POSIXt")
+        tz <- toupper(names(out))
+        tz[tz == "LOCAL"] <- ""
         out[dates] <- Map(lubridate::with_tz, out[dates], tz[dates])
         out
       }
@@ -510,26 +673,26 @@ spec_sql_write_table <- list(
   },
 
   #'
-  #' Mixing column types in the same table is supported.
   roundtrip_mixed = function(ctx, con) {
+    #' Mixing column types in the same table is supported.
     data <- list("a", 1L, 1.5)
-    data <- lapply(data, c, NA)
+    data <- map(data, c, NA)
     expanded <- expand.grid(a = data, b = data, c = data)
-    tbl_in_list <- lapply(
+    tbl_in_list <- map(
       seq_len(nrow(expanded)),
       function(i) {
-        as.data.frame(lapply(expanded[i, ], unlist, recursive = FALSE))
+        as.data.frame(map(expanded[i, ], unlist, recursive = FALSE))
       }
     )
 
-    lapply(tbl_in_list, test_table_roundtrip, con = con)
+    map(tbl_in_list, test_table_roundtrip, con = con)
   },
 
   #'
-  #' The `field.types` argument must be a named character vector with at most
-  #' one entry for each column.
-  #' It indicates the SQL data type to be used for a new column.
   roundtrip_field_types = function(ctx, con) {
+    #' The `field.types` argument must be a named character vector with at most
+    #' one entry for each column.
+    #' It indicates the SQL data type to be used for a new column.
     tbl_in <- data.frame(a = numeric(), b = character())
     #' If a column is missed from `field.types`, the type is inferred
     #' from the input data with [dbDataType()].
@@ -548,20 +711,19 @@ spec_sql_write_table <- list(
   },
 
   #'
-  #' The interpretation of [rownames] depends on the `row.names` argument,
-  #' see [sqlRownamesToColumn()] for details:
   write_table_row_names_false = function(ctx, con) {
+    #' The interpretation of [rownames] depends on the `row.names` argument,
+    #' see [sqlRownamesToColumn()] for details:
     #' - If `FALSE` or `NULL`, row names are ignored.
-    table_name <- random_table_name()
     for (row.names in list(FALSE, NULL)) {
-      with_remove_test_table(name = table_name, {
-        mtcars_in <- datasets::mtcars
-        dbWriteTable(con, table_name, mtcars_in, row.names = row.names)
-        mtcars_out <- check_df(dbReadTable(con, table_name, row.names = FALSE))
+      table_name <- random_table_name()
+      local_remove_test_table(con, table_name)
+      mtcars_in <- datasets::mtcars
+      dbWriteTable(con, table_name, mtcars_in, row.names = row.names)
+      mtcars_out <- check_df(dbReadTable(con, table_name, row.names = FALSE))
 
-        expect_false("row_names" %in% names(mtcars_out))
-        expect_equal_df(mtcars_out, unrowname(mtcars_in))
-      })
+      expect_false("row_names" %in% names(mtcars_out))
+      expect_equal_df(mtcars_out, unrowname(mtcars_in))
     }
   },
   #
@@ -583,14 +745,14 @@ spec_sql_write_table <- list(
     #'   even if the input data frame only has natural row names from 1 to `nrow(...)`.
     row.names <- TRUE
 
-    iris_in <- get_iris(ctx)
-    dbWriteTable(con, table_name, iris_in, row.names = row.names)
-    iris_out <- check_df(dbReadTable(con, table_name, row.names = FALSE))
+    penguins_in <- get_penguins(ctx)
+    dbWriteTable(con, table_name, penguins_in, row.names = row.names)
+    penguins_out <- check_df(dbReadTable(con, table_name, row.names = FALSE))
 
-    expect_true("row_names" %in% names(iris_out))
-    expect_true(all(rownames(iris_in) %in% iris_out$row_names))
-    expect_true(all(iris_out$row_names %in% rownames(iris_in)))
-    expect_equal_df(iris_out[names(iris_out) != "row_names"], iris_in)
+    expect_true("row_names" %in% names(penguins_out))
+    expect_true(all(rownames(penguins_in) %in% penguins_out$row_names))
+    expect_true(all(penguins_out$row_names %in% rownames(penguins_in)))
+    expect_equal_df(penguins_out[names(penguins_out) != "row_names"], penguins_in)
   },
   #
   write_table_row_names_na_exists = function(ctx, con, table_name) {
@@ -611,11 +773,11 @@ spec_sql_write_table <- list(
     #'   no extra column is created in the case of natural row names.
     row.names <- NA
 
-    iris_in <- get_iris(ctx)
-    dbWriteTable(con, table_name, iris_in, row.names = row.names)
-    iris_out <- check_df(dbReadTable(con, table_name, row.names = FALSE))
+    penguins_in <- get_penguins(ctx)
+    dbWriteTable(con, table_name, penguins_in, row.names = row.names)
+    penguins_out <- check_df(dbReadTable(con, table_name, row.names = FALSE))
 
-    expect_equal_df(iris_out, iris_in)
+    expect_equal_df(penguins_out, penguins_in)
   },
   #
   write_table_row_names_string_exists = function(ctx, con, table_name) {
@@ -638,18 +800,18 @@ spec_sql_write_table <- list(
     row.names <- "seq"
     #'   even if the input data frame only has natural row names.
 
-    iris_in <- get_iris(ctx)
-    dbWriteTable(con, table_name, iris_in, row.names = row.names)
-    iris_out <- check_df(dbReadTable(con, table_name, row.names = FALSE))
+    penguins_in <- get_penguins(ctx)
+    dbWriteTable(con, table_name, penguins_in, row.names = row.names)
+    penguins_out <- check_df(dbReadTable(con, table_name, row.names = FALSE))
 
-    expect_true("seq" %in% names(iris_out))
-    expect_true(all(iris_out$seq %in% rownames(iris_in)))
-    expect_true(all(rownames(iris_in) %in% iris_out$seq))
-    expect_equal_df(iris_out[names(iris_out) != "seq"], iris_in)
+    expect_true("seq" %in% names(penguins_out))
+    expect_true(all(penguins_out$seq %in% rownames(penguins_in)))
+    expect_true(all(rownames(penguins_in) %in% penguins_out$seq))
+    expect_equal_df(penguins_out[names(penguins_out) != "seq"], penguins_in)
   },
   #
+  #'
   write_table_row_names_default = function(ctx, con, table_name) {
-    #'
     #' The default is `row.names = FALSE`.
     mtcars_in <- datasets::mtcars
     dbWriteTable(con, table_name, mtcars_in)
@@ -668,8 +830,15 @@ test_table_roundtrip <- function(...) {
   test_table_roundtrip_one(..., .add_na = "below")
 }
 
-test_table_roundtrip_one <- function(con, tbl_in, tbl_expected = tbl_in, transform = identity,
-                                     name = NULL, field.types = NULL, use_append = FALSE, .add_na = "none") {
+test_table_roundtrip_one <- function(
+    con,
+    tbl_in,
+    tbl_expected = tbl_in,
+    transform = identity,
+    name = NULL,
+    field.types = NULL,
+    use_append = FALSE,
+    .add_na = "none") {
   force(tbl_expected)
   if (.add_na == "above") {
     tbl_in <- add_na_above(tbl_in)
